@@ -67,6 +67,7 @@ class signal_block(object):
 
     For instance:
 
+        >>> from kiwi.ui.views import SlaveView
         >>> class MyView(SlaveView):
         ...     @signal_block('money.changed')
         ...     def update_money(self):
@@ -94,13 +95,12 @@ class signal_block(object):
                 widget = getattr(view, name, None)
                 if widget is None:
                     raise TypeError("Unknown widget %s in view " % name)
-                view.handler_block(widget, signal)
+                view.handler_block(name, signal)
 
             retval = func(view, *args, **kwargs)
 
             for name, signal in self.signals:
-                widget = getattr(view, name, None)
-                view.handler_unblock(widget, signal)
+                view.handler_unblock(name, signal)
 
             return retval
         wrapper.__name__ = func.__name__
