@@ -23,6 +23,7 @@
 
 """Environment helpers: path mangling and resource management"""
 
+import commands
 import errno
 import gettext
 import imp
@@ -68,11 +69,11 @@ class Environment:
         # "glade" resources
         self._add_extensions("glade", ".bz2", ".gz")
 
+        self._is_egg = sys.argv[0].endswith('.egg')
+
         self._add_resource_variable("glade", "KIWI_GLADE_PATH")
         self._add_resource_variable("image", "KIWI_IMAGE_PATH")
 
-        self._is_egg = sys.argv[0].endswith('.egg')
-
     def get_root(self):
         return self._root
 
@@ -97,8 +98,11 @@ class Environment:
         return self._resources[resource]
 
     def add_resource(self, resource, path):
-        if self._is_egg:
-            return
+        # XXX. This is not as simple as just checkign if the file is a egg. We
+        # need to be sure that the current library is a egg itself, and not the
+        # current running program.
+        #if self._is_egg:
+        #    return
         path = os.path.join(self._root, path)
 
         if not os.path.isdir(path):
@@ -195,23 +199,20 @@ class Library(Environment):
     >>> from kiwi.environ import Library
     >>> lib = Library('kiwi')
     >>> if lib.uninstalled:
-    >>>     lib.add_global_resource('glade', 'glade')
-    >>>     lib.add_global_resource('pixmap', 'pixmaps')
+    ...     lib.add_global_resource('glade', 'glade')
+    ...     lib.add_global_resource('pixmap', 'pixmaps')
 
     which is combined with the following class in setup.py:
 
-    >>> from kiwi.dist import InstallLib
-    >>> class InstallLib(TemplateInstallLib):
-    >>>    name = 'kiwi'
-    >>>    global_resources = dict(glade='$datadir/glade',
-    >>>                            pixmap='$datadir/pixmaps')
-    >>>
-    >>> setup(...,
-    >>>       data_files=[('share/kiwi/glade',
-    >>>                   listfiles('glade', '*.glade')),
-    >>>                   ('share/kiwi/pixmaps',
-    >>>                   listfiles('pixmaps', '*.png')),
-    >>>       cmdclass=dict(install_lib=InstallLib))
+    >>> from kiwi.dist import KiwiInstallLib, setup, listfiles
+    >>> class InstallLib(KiwiInstallLib):
+    ...     name = 'kiwi'
+    ...     global_resources = dict(glade='$datadir/glade',
+    ...                             pixmap='$datadir/pixmaps')
+    >>> setup(data_files=[
+    ...           ('share/kiwi/glade', listfiles('glade', '*.glade')),
+    ...           ('share/kiwi/pixmaps', listfiles('pixmaps', '*.png'))],
+    ...       cmdclass=dict(install_lib=InstallLib))
 
     It may seems like a bit of work, but this is everything that's needed
     for kiwi to figure out how to load resources when installed and when
@@ -383,6 +384,9 @@ class Library(Environment):
 
     def get_revision(self):
         if self.uninstalled:
+            status, output = commands.getstatusoutput('git rev-parse --short HEAD')
+            if status == 0:
+                return output
             revision = os.path.join(
                 self._root, '.bzr', 'branch', 'last-revision')
             try:
@@ -406,8 +410,8 @@ class Application(Library):
     >>> from kiwi.environ import Application
     >>> app = Application('gnomovision')
     >>> if app.uninstalled:
-    >>>     app.add_global_resource('glade', 'glade')
-    >>>     app.add_global_resource('pixmap', 'pixmaps')
+    ...     app.add_global_resource('glade', 'glade')
+    ...     app.add_global_resource('pixmap', 'pixmaps')
 
     If you want to do translations, you also need to do the following:
 
