@@ -25,7 +25,31 @@ import glib
 import gobject
 import gtk
 
-from kiwi.ui.gadgets import gdk_color_to_string
+from kiwi.ui.gadgets import gdk_color_to_string, draw_editable_border
+
+
+class EditableTextRenderer(gtk.CellRendererText):
+    """Adds a border to an editable text so it can be told apart from not-editable."""
+
+    def do_render(self, drawable, widget, background_area, cell_area, expose_area, flags):
+        draw_editable_border(widget, drawable, cell_area)
+        gtk.CellRendererText.do_render(self, drawable, widget, background_area,
+                                       cell_area, expose_area, flags)
+
+
+gobject.type_register(EditableTextRenderer)
+
+
+class EditableSpinRenderer(gtk.CellRendererSpin):
+    """Adds a border to an editable spin so it becomes easy to see it is editable."""
+
+    def do_render(self, drawable, widget, background_area, cell_area, expose_area, flags):
+        draw_editable_border(widget, drawable, cell_area)
+        gtk.CellRendererText.do_render(self, drawable, widget, background_area,
+                                       cell_area, expose_area, flags)
+
+
+gobject.type_register(EditableSpinRenderer)
 
 
 class ComboDetailsCellRenderer(gtk.CellRenderer):
@@ -42,7 +66,12 @@ class ComboDetailsCellRenderer(gtk.CellRenderer):
     label = gobject.property(type=str, default="")
     data = gobject.property(type=object)
 
-    def __init__(self):
+    def __init__(self, use_markup=False):
+        """
+        :param use_markup: wheter all strings we send in are already
+          in pango markup
+        """
+        self.use_markup = use_markup
         label = gtk.Label()
         self._label_layout = label.create_pango_layout('')
         self._details_callback = None
@@ -59,9 +88,14 @@ class ComboDetailsCellRenderer(gtk.CellRenderer):
         # Center the label
         y_offset = cell_area.height / 2 - height / 2
 
+        if flags & gtk.CELL_RENDERER_SELECTED:
+            state = gtk.STATE_SELECTED
+        else:
+            state = gtk.STATE_ACTIVE
+
         # Draws label
         widget.style.paint_layout(window,
-                                  gtk.STATE_ACTIVE, False,
+                                  state, False,
                                   cell_area, widget, "",
                                   cell_area.x + x_offset,
                                   cell_area.y + y_offset,
@@ -103,15 +137,19 @@ class ComboDetailsCellRenderer(gtk.CellRenderer):
     else:
         do_render = do_render_gtk2
 
+    def _escape(self, text):
+        if not self.use_markup:
+            text = glib.markup_escape_text(text)
+        return text
+
     def do_get_size(self, widget, cell_area):
         if self._details_callback:
             details = self._details_callback(self.data)
-            mark_up = '%s\n<span foreground="%s">%s</span>'
+            mark_up = '%s\n%s'
             color = gdk_color_to_string(widget.style.fg[gtk.STATE_NORMAL])
-            text = mark_up % (self.label, color,
-                              glib.markup_escape_text(details))
+            text = mark_up % (self.label, color, self._escape(details))
         else:
-            text = glib.markup_escape_text(self.label)
+            text = self._escape(self.label)
 
         self._label_layout.set_markup(text)
         width, height = self._label_layout.get_pixel_size()
