@@ -276,11 +276,40 @@ class TextField(Field):
                 widget.set_mask(self.input_mask)
         else:
             widget = ProxyLabel()
+            # This label should be left aligned.
+            widget.set_alignment(0, 0.5)
+
         return widget
 
 gobject.type_register(TextField)
 
 
+class PasswordField(TextField):
+    """
+    I am a password field with one line, editable by the user,
+    rendered as an entry with invisible characters.
+    """
+    def build_widget(self):
+        widget = TextField.build_widget(self)
+        if self.editable:
+            widget.set_visibility(False)
+
+        return widget
+
+
+class IntegerField(Field):
+    """
+    I am a numeric field with one line, editable by the user,
+    rendered as an entry that allows only integer numbers
+    """
+
+    widget_data_type = int
+
+    def build_widget(self):
+        widget = ProxyEntry()
+        return widget
+
+
 class ChoiceField(Field):
     """
     I am a field representing a set of choices,
@@ -477,7 +506,7 @@ class BasicForm(SlaveDelegate):
         if not view:
             raise TypeError(
                 "Form %r requires a view, not %r" % (
-                self.__class__.__name__, view, ))
+                    self.__class__.__name__, view, ))
         self.proxy = None
         self.main_view = view
         # Just a simple GtkBin
