@@ -46,6 +46,7 @@ from kiwi.log import Logger
 from kiwi.python import enum, slicerange
 from kiwi.utils import gsignal, type_register
 from kiwi.ui.widgets.contextmenu import ContextMenu
+from kiwi.ui.cellrenderer import EditableTextRenderer, EditableSpinRenderer
 
 _ = lambda m: gettext.dgettext('kiwi', m)
 
@@ -281,6 +282,9 @@ class Column(gobject.GObject):
 
     def __repr__(self):
         namespace = self.__dict__.copy()
+        # Dont know why yet, but we have to remove the compare key, otherwise,
+        # this may go into an infinite recursion
+        namespace.pop('compare')
         return "<%s: %s>" % (self.__class__.__name__, namespace)
 
     def _get_data_type(self):
@@ -399,7 +403,7 @@ class Column(gobject.GObject):
         return treeview_column
 
     def create_renderer(self, model):
-        """Gusses which CellRenderer we should use for a given type.
+        """Guesses which CellRenderer we should use for a given type.
         It also set the property of the renderer that depends on the model,
         in the renderer.
         """
@@ -444,7 +448,7 @@ class Column(gobject.GObject):
                                  model, self.attribute, self)
             prop = 'model'
         elif issubclass(data_type, number) and self.spin_adjustment:
-            renderer = gtk.CellRendererSpin()
+            renderer = EditableSpinRenderer()
             if not self.editable:
                 raise TypeError("spin_adjustment columns must be editable")
 
@@ -456,16 +460,18 @@ class Column(gobject.GObject):
         elif issubclass(data_type, (datetime.date, datetime.time,
                                     basestring, number,
                                     currency)):
-            renderer = gtk.CellRendererText()
             if self.use_markup:
                 prop = 'markup'
             else:
                 prop = 'text'
             if self.editable:
+                renderer = EditableTextRenderer()
                 renderer.set_property('editable', True)
                 renderer.connect('edited', self._on_renderer_text__edited,
                                  model, self.attribute, self,
                                  self.from_string)
+            else:
+                renderer = gtk.CellRendererText()
             if self.width_chars != -1:
                 renderer.set_property('width-chars', self.width_chars)
         else:
@@ -537,6 +543,10 @@ class Column(gobject.GObject):
         obj = row[COL_MODEL]
         data = column.get_attribute(obj, column.attribute, None)
         text = column.as_string(data, obj)
+
+        if self._objectlist.cell_data_func and obj is not empty_marker:
+            text = self._objectlist.cell_data_func(self, renderer, obj, text)
+
         renderer.set_property(renderer_prop, text)
 
         if column.renderer_func:
@@ -599,6 +609,17 @@ class Column(gobject.GObject):
         except ValidationError:
             return
 
+        # FIXME: There's something wrong on the spinbutton that lets the user
+        # type and confirm an invalid value (it's not possible to get using
+        # the mouse, just the keyboard). Mimic the normal spin behaviour that
+        # is to adjust the value to [lower, upper] range.
+        upper = self.spin_adjustment.get_upper()
+        lower = self.spin_adjustment.get_lower()
+        if value_model > upper:
+            value_model = upper
+        if value_model < lower:
+            value_model = lower
+
         setattr(obj, attr, value_model)
         self._objectlist.emit('cell-edited', obj, attr)
 
@@ -643,11 +664,10 @@ class Column(gobject.GObject):
         data_type = self.data_type
         if data is None and data_type != gdk.Pixbuf:
             text = ''
-        elif self.format_func:
-            if self.format_func_data is not None:
-                text = self.format_func(obj, self.format_func_data)
-            else:
-                text = self.format_func(data)
+        elif self.format_func and self.format_func_data is not None and obj:
+            text = self.format_func(obj, self.format_func_data)
+        elif self.format_func and self.format_func_data is None:
+            text = self.format_func(data)
         elif (self.format or
               data_type == float or
               data_type == decimal.Decimal or
@@ -719,57 +739,6 @@ class SequentialColumn(Column):
                             (renderer, renderer_prop))
 
 
-class SearchColumn(Column):
-    """
-    I am a column that should be used in conjunction with
-    :class:`kiwi.ui.search.SearchSlaveDelegate`
-
-    :param long_title: The title to display in the combo for this field.
-                       This is usefull if you need to display a small
-                       description on the column header, but still want a full
-                       description on the advanced search.
-    :param valid_values: This should be a list of touples (display value, db
-                         value). If provided, then a combo with only this
-                         values will be shown, instead of a free text entry.
-    :param search_attribute: Use this if the name of the db column that should
-                             be searched is different than the attribute of
-                             the model.
-    """
-
-    #: overrides the function that generates the query to process the search
-    search_func = gobject.property(type=object, default=None)
-
-    #: names the search interface differently from the column
-    search_label = gobject.property(type=object, default=None)
-
-    #: use the query on the *having* part instead of the *where*'s on
-    use_having = gobject.property(type=bool, default=False)
-
-    def __init__(self, attribute, title=None, data_type=None,
-                 long_title=None, valid_values=None, search_attribute=None,
-                 **kwargs):
-        """
-        """
-        self.long_title = long_title
-        self.valid_values = valid_values
-        self.search_attribute = search_attribute
-        self.sensitive = True
-
-        Column.__init__(self, attribute, title, data_type, **kwargs)
-
-        search_func = kwargs.get('search_func')
-        if search_func and not callable(search_func):
-            raise TypeError("search_func must be callable")
-
-    def get_search_label(self):
-        """Get the search label for this column.
-          This is normally used when constructing a search filter for this
-          column.
-          :returns: the search label
-        """
-        return self.search_label or self.long_title or self.title
-
-
 class ColoredColumn(Column):
     """
     I am a column which can colorize the text of columns under
@@ -924,23 +893,23 @@ class ObjectList(gtk.HBox):
     Items in an ObjectList is stored in objects. Each row represents an object
     and each column represents an attribute in the object.
     The column description object must be a subclass of :class:`Column`.
-    Simple example
+    Simple example::
 
-    >>> class Fruit:
-    >>>    pass
+        >>> class Fruit:
+        ...    pass
 
-    >>> apple = Fruit()
-    >>> apple.name = 'Apple'
-    >>> apple.description = 'Worm house'
+        >>> apple = Fruit()
+        >>> apple.name = 'Apple'
+        >>> apple.description = 'Worm house'
 
-    >>> banana = Fruit()
-    >>> banana.name = 'Banana'
-    >>> banana.description = 'Monkey food'
+        >>> banana = Fruit()
+        >>> banana.name = 'Banana'
+        >>> banana.description = 'Monkey food'
 
-    >>> fruits = ObjectList([Column('name'),
-    >>>                      Column('description')])
-    >>> fruits.append(apple)
-    >>> fruits.append(banana)
+        >>> fruits = ObjectList([Column('name'),
+        ...                      Column('description')])
+        >>> fruits.append(apple)
+        >>> fruits.append(banana)
 
     Signals
     =======
@@ -1145,8 +1114,8 @@ class ObjectList(gtk.HBox):
             item = self._model[arg][COL_MODEL]
         elif isinstance(arg, slice):
             model = self._model
-            return [model[item][COL_MODEL]
-                    for item in slicerange(arg, len(self._model))]
+            return [model[i][COL_MODEL]
+                    for i in slicerange(arg, len(self._model))]
         else:
             raise TypeError("argument arg must be int, gtk.Treeiter or "
                             "slice, not %s" % type(arg))
@@ -1288,7 +1257,11 @@ class ObjectList(gtk.HBox):
         # do nothing if empty list or None provided
         model = self._model
         if clear:
-            if not instances:
+            if hasattr(instances, 'is_empty'):
+                is_empty = instances.is_empty()
+            else:
+                is_empty = not bool(instances)
+            if is_empty:
                 self.unselect_all()
                 self.clear()
                 return
@@ -1841,6 +1814,8 @@ class ObjectList(gtk.HBox):
             raise TypeError("You can only select multiple items with"
                             "selection mode set to gtk.SELECTION_MULTIPLE")
 
+        # FIXME: This is not working for multiple selections. Only the last item
+        # in the list remains selected
         for instance in instances:
             if not instance in self._iters:
                 raise ValueError("instance %s is not in the list" % repr(instance))
@@ -2429,12 +2404,24 @@ class SummaryLabel(ListLabel):
     data types and I will complain if you give me something else."""
 
     def __init__(self, klist, column, label=_('Total:'), value_format='%s',
-                 font_desc=None):
+                 font_desc=None, data_func=None):
+        """
+        :param data_func: a function or method that can be used to determine
+        the signedness of the object being summed. Returns a bool, ``True``
+        means positive, ``False`` means negative.
+        """
+        if data_func and not callable(data_func):
+            raise ValueError("data_func must be callable, not %r"
+                             % (data_func,))
+
         ListLabel.__init__(self, klist, column, label, value_format, font_desc)
         if not issubclass(self._column.data_type, number):
             raise TypeError("data_type of column must be a number, not %r",
                             self._column.data_type)
         klist.connect('cell-edited', self._on_klist__cell_edited)
+
+        self._data_func = data_func
+
         self.update_total()
 
     # Public API
@@ -2445,8 +2432,13 @@ class SummaryLabel(ListLabel):
         attr = column.attribute
         get_attribute = column.get_attribute
 
-        value = sum([get_attribute(obj, attr, 0) or 0 for obj in self._klist],
-                    column.data_type('0'))
+        value = column.data_type('0')
+        for obj in self._klist:
+            attr_value = get_attribute(obj, attr, 0) or 0
+            if self._data_func and not self._data_func(obj):
+                value -= attr_value
+            else:
+                value += attr_value
 
         self.set_value(column.as_string(value))
 
