@@ -107,6 +107,13 @@ class Proxy:
         if value is ValueUnset:
             return
 
+        # Dont update the model if the widget is not sensitive, or if it is not
+        # editable.
+        if not widget.is_sensitive():
+            return
+        if isinstance(widget, gtk.Editable) and not widget.get_editable():
+            return
+
         model = self.model
         # XXX: One day we might want to queue and unique updates?
         if hasattr(model, "block_proxy"):
@@ -342,15 +349,15 @@ class Proxy:
                 raise TypeError(
                     "attribute %s of model %r requires a value of "
                     "type %s, not %s" % (
-                    attribute, self._model,
-                    data_type.__name__,
-                    value_type.__name__))
+                        attribute, self._model,
+                        data_type.__name__,
+                        value_type.__name__))
 
         if block:
             block_widget(widget)
-            self._view.handler_block(widget)
+            self._view.handler_block(attribute)
             widget.update(value)
-            self._view.handler_unblock(widget)
+            self._view.handler_unblock(attribute)
             unblock_widget(widget)
         else:
             widget.update(value)
