@@ -180,6 +180,7 @@ class ProxyWidgetMixin(object):
 
 VALIDATION_ICON_WIDTH = 16
 MANDATORY_ICON = gtk.STOCK_EDIT
+MANDATORY_COLOR = '#fcf6c6'
 
 VALIDATION_PNG = """iVBORw0KGgoAAAANSUhEUgAAABEAAAARCAYAAAA7bUf6AAAABGdBTUEAANbY1E9YMgAAABl0RVh0
 U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAFGSURBVHjaYmRAA/8ZGOyBlAMDbnCAkYHh
@@ -223,6 +224,8 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
         self._fade = FadeOut(self)
         self._fade.connect('color-changed', self._on_fadeout__color_changed)
         self.connect('notify::mandatory', self._on_notify__mandatory)
+        self.connect('notify::sensitive', self._on_notify__sensitive)
+        self.connect('notify::visible', self._on_notify__visible)
 
     # Override in subclass
 
@@ -259,12 +262,15 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
         # when forcing the validation
         if not force and (not self.get_property('visible') or
                           not self.get_property('sensitive')):
+            self._set_pixbuf(None)
             return ValueUnset
 
         try:
             data = self.read()
             log.debug('Read %r for %s' % (data, self.model_attribute))
 
+            self.validate_value(data)
+
             # check if we should draw the mandatory icon
             # this need to be done before any data conversion because we
             # we don't want to end drawing two icons
@@ -291,6 +297,16 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
             self.set_invalid(str(e))
             return ValueUnset
 
+    def validate_value(self, value):
+        """Extra validation for *value*
+
+        :meth:`.validate` will call this as soon as it calls :meth:`.read`
+        with it's return value. This should raise ValidationError if the
+        value should be considered invalid
+
+        :raises: :exc:`kiwi.datatypes.ValidationError`
+        """
+
     def set_valid(self):
         """Changes the validation state to valid, which will remove icons and
         reset the background color
@@ -300,7 +316,9 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
         self._set_valid_state(True)
 
         self._fade.stop()
-        self.set_pixbuf(None)
+        self._set_pixbuf(None)
+        style = gtk.widget_get_default_style()
+        self.update_background(style.base[gtk.STATE_NORMAL])
 
     def set_invalid(self, text=None, fade=True):
         """Changes the validation state to invalid.
@@ -317,7 +335,7 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
             text = _("'%s' is not a valid value for this field") % self.read()
 
         if not fade:
-            self.set_pixbuf(_load_error_icon())
+            self._set_pixbuf(_load_error_icon())
             self.update_background(gtk.gdk.color_parse(self._fade.ERROR_COLOR))
             return
 
@@ -326,7 +344,7 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
         # (which removes this timeout) is called as soon as the user
         # types valid data.
         def done(fadeout, c):
-            self.set_pixbuf(_load_error_icon())
+            self._set_pixbuf(_load_error_icon())
             self.queue_draw()
             fadeout.disconnect(c.signal_id)
 
@@ -336,7 +354,7 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
         c.signal_id = self._fade.connect('done', done, c)
 
         if self._fade.start(self.get_background()):
-            self.set_pixbuf(None)
+            self._set_pixbuf(None)
 
         # If you try to set the tooltip before the icon in gtk.Entry, a
         # segfault happens.
@@ -352,6 +370,7 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
             self._draw_stock_icon(MANDATORY_ICON)
             self.set_tooltip(_('This field is mandatory'))
             self._fade.stop()
+            self.update_background(gtk.gdk.color_parse(MANDATORY_COLOR))
             valid = False
         else:
             valid = True
@@ -360,6 +379,17 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
 
     # Private
 
+    def _set_pixbuf(self, pixbuf):
+        # Even though self.validate will call this with None when not
+        # visible/sensitive, passing force=True to it will make it validate
+        # the widget. That's ok, since it will set self._valid right, but
+        # we don't want to draw a mandatory/validation_error icon on
+        # an insensitive widget
+        if not self.get_sensitive():
+            pixbuf = None
+
+        self.set_pixbuf(pixbuf)
+
     def _set_valid_state(self, state):
         """Updates the validation state and emits a signal if it changed"""
         # FIXME: This should not happen, but somehow, model_attribute is being
@@ -375,7 +405,7 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
 
     def _draw_stock_icon(self, stock_id):
         icon = self.render_icon(stock_id, gtk.ICON_SIZE_MENU)
-        self.set_pixbuf(icon)
+        self._set_pixbuf(icon)
         self.queue_draw()
 
     # Callbacks
@@ -385,3 +415,9 @@ class ValidatableProxyWidgetMixin(ProxyWidgetMixin):
 
     def _on_notify__mandatory(self, obj, pspec):
         self.validate()
+
+    def _on_notify__sensitive(self, obj, pspec):
+        self.validate()
+
+    def _on_notify__visible(self, obj, pspec):
+        self.validate()
