@@ -1,7 +1,7 @@
 #
 # Kiwi: a Framework and Enhanced Widgets for Python
 #
-# Copyright (C) 2001-2012 Async Open Source
+# Copyright (C) 2001-2013 Async Open Source
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -31,23 +31,24 @@ are the base of Delegates and Proxies.
 """
 
 import os
-import re
 import string
 
 import gobject
 import gtk
 from gtk import gdk
 
-from kiwi import ValueUnset
-from kiwi.datatypes import ValidationError
 from kiwi.environ import environ
 from kiwi.interfaces import IValidatableProxyWidget
 from kiwi.log import Logger
 from kiwi.python import namedAny
+from kiwi.ui.signal import GladeSignalBroker, SignalBroker, SignalProxyObject
 from kiwi.utils import gsignal, type_register
 from kiwi.ui.gadgets import quit_if_last, register_notebook_shortcuts
 from kiwi.ui.proxy import Proxy
 
+# backwards compatibility
+SignalProxyObject
+
 log = Logger('kiwi.view')
 
 validation_log = Logger('kiwi.validation')
@@ -81,136 +82,6 @@ if hasattr(gtk, 'Progress'):
 color_red = gdk.color_parse('red')
 color_black = gdk.color_parse('black')
 
-#
-# Signal brokers
-#
-
-method_regex = re.compile(r'^(on|after)_(\w+)__(\w+)$')
-
-
-class SignalBroker(object):
-    def __init__(self, view, controller=None):
-        if controller is None:
-            controller = view
-        methods = self._get_all_methods(controller)
-        self._do_connections(view, methods)
-
-    def _get_all_methods(self, controller, klass=None):
-        klass = klass or controller.__class__
-        # Very poor simulation of inheritance, but WFM(tm)
-        classes = [klass]
-        # Collect bases for class, using a pretty evil recursion
-        for klass in classes:
-            map(classes.append, klass.__bases__)
-        # Order bases so that the class itself is the last one referred to
-        # in the loop. This guarantees that the inheritance ordering for the
-        # methods is preserved.
-        classes.reverse()
-        methods = {}
-        for c in classes:
-            for name in c.__dict__.keys():
-                # Need to use getattr() to ensure we get bound methods
-                try:
-                    methods[name] = getattr(controller, name)
-                except AttributeError:
-                    continue
-        return methods
-
-    def _do_connections(self, view, methods):
-        """This method allows subclasses to add more connection mechanism"""
-        self._autoconnect_by_method_name(view, methods)
-
-    def _autoconnect_by_method_name(self, view, methods):
-        """
-        Offers autoconnection of widget signals based on function names.
-        You simply need to define your controller method in the format::
-
-            def on_widget_name__signal_name(self, widget):
-
-        In other words, start the method by "on_", followed by the
-        widget name, followed by two underscores ("__"), followed by the
-        signal name. Note: If more than one double underscore sequences
-        are in the string, the last one is assumed to separate the
-        signal name.
-        """
-        self._autoconnected = {}
-
-        for fname in methods.keys():
-            # `on_x__y' has 7 chars and is the smallest possible handler
-            if len(fname) < 7:
-                continue
-            match = method_regex.match(fname)
-            if match is None:
-                continue
-            on_after, w_name, signal = match.groups()
-            widget = getattr(view, w_name, None)
-            if widget is None:
-                raise AttributeError("couldn't find widget %s in %s"
-                                     % (w_name, view))
-            if not isinstance(widget, gobject.GObject):
-                raise AttributeError("%s (%s) is not a widget or an action "
-                                     "and can't be connected to"
-                                     % (w_name, widget))
-            # Must use getattr; using the class method ends up with it
-            # being called unbound and lacking, thus, "self".
-            try:
-                if on_after == 'on':
-                    signal_id = widget.connect(signal, methods[fname])
-                elif on_after == 'after':
-                    signal_id = widget.connect_after(signal, methods[fname])
-                else:
-                    raise AssertionError
-            except TypeError:
-                raise TypeError("Widget %s doesn't provide a signal %s" % (
-                                widget.__class__, signal))
-            self._autoconnected.setdefault(widget, []).append((
-                signal, signal_id))
-
-    def handler_block(self, widget, signal_name):
-        signals = self._autoconnected
-        if not widget in signals:
-            return
-
-        for signal, signal_id in signals[widget]:
-            if signal_name is None or signal == signal_name:
-                widget.handler_block(signal_id)
-
-    def handler_unblock(self, widget, signal_name):
-        signals = self._autoconnected
-        if not widget in signals:
-            return
-
-        for signal, signal_id in signals[widget]:
-            if signal_name is None or signal == signal_name:
-                widget.handler_unblock(signal_id)
-
-    def disconnect_autoconnected(self):
-        for widget, signals in self._autoconnected.items():
-            for signal in signals:
-                widget.disconnect(signal[1])
-
-
-class GladeSignalBroker(SignalBroker):
-    def _do_connections(self, view, methods):
-        super(GladeSignalBroker, self)._do_connections(view, methods)
-        self._connect_glade_signals(view, methods)
-
-    def _connect_glade_signals(self, view, methods):
-        # mainly because the two classes cannot have a common base
-        # class. studying the class layout carefully or using
-        # composition may be necessary.
-
-        # called by framework.basecontroller. takes a controller, and
-        # creates the dictionary to attach to the signals in the tree.
-        if not methods:
-            raise AssertionError("controller must be provided")
-
-        dict = {}
-        for name, method in methods.items():
-            if callable(method):
-                dict[name] = method
-        view._glade_adaptor.signal_autoconnect(dict)
-
 
 class SlaveView(gobject.GObject):
     """
@@ -339,12 +210,19 @@ class SlaveView(gobject.GObject):
 
         return toplevel
 
+    def get_resource_string(self, domain, resource, name):
+        """ This method might be overriden by subclasses in order to use it's own
+        resource_string function (like accessing a resource from inside an egg)
+        """
+        return environ.get_resource_string(domain, resource, name)
+
     def get_glade_adaptor(self):
         """Special init code that subclasses may want to override."""
         if not self.gladefile:
             return
 
-        glade_adaptor = _open_glade(self, self.gladefile, self.domain)
+        glade_adaptor = _open_glade(self, self.gladefile, self.domain,
+                                    self.get_resource_string)
 
         container_name = self.toplevel_name
         if not container_name:
@@ -379,7 +257,7 @@ class SlaveView(gobject.GObject):
             raise TypeError(
                 "%r is attached to %r, you need to specify "
                 "another holder name" % (
-                self._forms[holder_name], holder_name))
+                    self._forms[holder_name], holder_name))
         # To avoid a cyclic dependency between forms & views
         from kiwi.ui.forms import BasicForm
         form = BasicForm(self)
@@ -797,22 +675,20 @@ class SlaveView(gobject.GObject):
             else:
                 widget.connect(signal, handler)
 
-    def disconnect_autoconnected(self):
-        """
-        Disconnect handlers previously connected with
-        autoconnect_signals()"""
-        self._broker.disconnect_autoconnected()
-
-    def handler_block(self, widget, signal_name=None):
+    def handler_block(self, object_name, signal_name=None):
         # XXX: Warning, or bail out?
         if not self._broker:
             return
-        self._broker.handler_block(widget, signal_name)
+        signal_proxy = getattr(self.__class__, object_name, None)
+        if signal_proxy is not None:
+            signal_proxy.handler_block(self, signal_name)
 
-    def handler_unblock(self, widget, signal_name=None):
+    def handler_unblock(self, object_name, signal_name=None):
         if not self._broker:
             return
-        self._broker.handler_unblock(widget, signal_name)
+        signal_proxy = getattr(self.__class__, object_name, None)
+        if signal_proxy is not None:
+            signal_proxy.handler_unblock(self, signal_name)
 
     #
     # Proxies
@@ -839,6 +715,7 @@ class SlaveView(gobject.GObject):
             model and model.__class__.__name__))
 
         widgets = widgets or self.widgets
+        proxy_widgets = []
 
         for widget_name in widgets:
             widget = getattr(self, widget_name, None)
@@ -856,22 +733,25 @@ class SlaveView(gobject.GObject):
                 raise AssertionError("%r does not have a validation-changed "
                                      "signal." % widget)
 
-            if (widget.props.mandatory and
-                widget.props.sensitive and
-                widget.props.visible):
-                try:
-                    value = widget.read()
-                    empty = value is ValueUnset or value is None
-                    validation_log.info("%s: %s=%r (initial)" % (
-                        self.__class__.__name__, widget_name, value))
-                    if empty:
-                        self._validation[widget_name] = False
-                # FIXME: This is probably wrong, but we get improperly placed
-                #        thousand operators in other places that has to be fixed first
-                except ValidationError:
-                    pass
+            proxy_widgets.append((widget_name, widget))
+
         proxy = Proxy(self, model, widgets)
         self._proxies.append(proxy)
+
+        for widget_name, widget in proxy_widgets:
+            # Do not store validation value for invisible/insensitive widgets.
+            # If they turn visible/sensitive, _on_child__validation_changed
+            # will handle that
+            if (not widget.get_property('visible') or
+                not widget.get_property('sensitive')):
+                continue
+
+            # Proxy.__init__ will call widget.validate(force=True), so we
+            # can use rely on widget.is_valid() here
+            self._validation[widget_name] = widget.is_valid()
+            validation_log.info("%s: %s=%r (initial)" % (
+                self.__class__.__name__, widget_name, widget.is_valid()))
+
         return proxy
 
     #
@@ -978,7 +858,8 @@ class BaseView(SlaveView):
         if not self.gladefile:
             return
 
-        return _open_glade(self, self.gladefile, self.domain)
+        return _open_glade(self, self.gladefile, self.domain,
+                           self.get_resource_string)
 
     #
     # Hook for keypress handling
@@ -1108,7 +989,7 @@ def _get_builder():
     return BuilderWidgetTree
 
 
-def _open_glade(view, gladefile, domain):
+def _open_glade(view, gladefile, domain, resource_str_func):
     if not gladefile:
         raise ValueError("A gladefile wasn't provided.")
     elif not isinstance(gladefile, basestring):
@@ -1116,7 +997,7 @@ def _open_glade(view, gladefile, domain):
             "gladefile should be a string, found %s" % type(gladefile))
 
     if _glade_loader_func:
-        return _glade_loader_func(view, gladefile, domain)
+        return _glade_loader_func(view, gladefile, domain, resource_str_func)
 
     if gladefile.endswith('.ui'):
         directory = os.path.dirname(namedAny(view.__module__).__file__)
